-----------------------------------------------------------------------------

local AloftGlow = Aloft:NewModule("Glow", "AceEvent-2.0", "AceHook-2.1")
local SML = AceLibrary("LibSharedMedia-3.0")

AloftGlow.db = Aloft:AcquireDBNamespace("glow")
Aloft:RegisterDefaults("glow", "profile", {
	enable			= true,
	interval		= 0.2, -- update glow 5 times a second (nominally)
	width			= 140,
	height			= 45,
	offsetX			= 0,
	offsetY			= 0,
	override		= false,
	color			= { 1.0, 0.0, 0.0, 1.0 }, -- red
	style			= "SQUAREGLOW",
	blendMode		= "BLEND",
})

-----------------------------------------------------------------------------

local profile
local glowFramePool = { }
local unpack=unpack

local glowTexture =
{
	["HARD"]				= [[Interface\Addons\Aloft\Textures\Outline]],			-- TODO: research this [[]] notation
	["SOFT"]				= [[Interface\AddOns\Aloft\Textures\SoftOutline]],
	["EXTRASOFT"]			= [[Interface\AddOns\Aloft\Textures\ExtraSoftOutline]],
	["OVAL"]				= [[Interface\AddOns\Aloft\Textures\OvalOutline]],
	["OVALGLOW"]			= [[Interface\AddOns\Aloft\Textures\OvalGlow]],
	["SQUAREGLOW"]			= [[Interface\AddOns\Aloft\Textures\SquareGlow]],
	["PRTLSQUAREGLOW"]		= [[Interface\AddOns\Aloft\Textures\PartialSquareGlow]],
	["THRQTRSQUAREGLOW"]	= [[Interface\AddOns\Aloft\Textures\ThrQrtrSquareGlow]],
}

-----------------------------------------------------------------------------

function AloftGlow:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		self:SetupFrame(aloftData)
		--[[
		if (aloftData.glowFrame) then
			self:SetupGlowFrame(aloftData.glowFrame)
			self:PlaceGlowFrame(aloftData)
		end
		]]
	end
	--[[
	if profile.enable then
		for glowFrame in pairs(glowFramePool) do
			self:SetupGlowFrame(glowFrame)
		end
	end
	]]
end

-----------------------------------------------------------------------------

function AloftGlow:OnInitialize()
	profile = self.db.profile
end

function AloftGlow:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")

	self:UpdateAll()
end

function AloftGlow:OnDisable()
	self:UnregisterAllEvents()
	--self:UnhookAll()

	--[[
	for aloftData in Aloft:IterateVisibleNameplates() do
		self:ReleaseGlowFrame(aloftData)
	end
	]]
	-- TODO: put bGlowRegion back to its original form somehow?
end

function AloftGlow:OnNameplateShow(aloftData)
end

function AloftGlow:OnNameplateHide(aloftData)
end

-- TODO: watch for unitid changes, hook/unhook on that basis as well
function AloftGlow:SetupFrame(aloftData)
	local bGlowRegion = aloftData.bGlowRegion
	local healthBar = aloftData.healthBar

	bGlowRegion:SetTexture(glowTexture[profile.style])
	bGlowRegion:ClearAllPoints()
	bGlowRegion:SetPoint("TOPLEFT", healthBar "TOPLEFT", profile.offsetX, profile.offsetY)
	bGlowRegion:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", profile.offsetX, profile.offsetY)
	--bGlowRegion:SetPoint("CENTER", this, "CENTER", profile.offsetX, profile.offsetY)

	bGlowRegion:SetWidth(profile.width)
	bGlowRegion:SetHeight(profile.height)

	if (profile.override) then
		bGlowRegion:SetVertexColor(unpack(profile.color)) -- override the underlying blizzard color(s)
	end

	bGlowRegion:SetBlendMode(profile.blendMode)

	-- zap the Blizzard default flash region, we have no way to control it, it does not seem even to generate events; it is replaced here if the user wants it
	--bGlowRegion:SetTexture("")
	--[[
	bGlowRegion:SetTexture(glowTexture[profile.style])
	bGlowRegion:SetBlendMode(profile.blendMode)

	bGlowRegion:ClearAllPoints()
	bGlowRegion:SetWidth(profile.width)
	bGlowRegion:SetHeight(profile.height)
	bGlowRegion:SetPoint("CENTER", aloftData.healthBar, "CENTER", profile.offsetX, profile.offsetY)

	if (profile.override) then
		bGlowRegion:SetVertexColor(unpack(profile.color)) -- override the underlying blizzard color(s)
	end
	]]

	-- NOTE: the WoW client is in charge of showing/hiding the glow region; we just hijacked it to the extent required to alter its appearance

	--local glowFrame
	--if (profile.enable) then
	--	glowFrame = self:AcquireGlowFrame(aloftData)
	--	self:CleanupGlowFrame(aloftData)

		if (not self:IsHooked(nameplateFrame, "OnUpdate")) then
			self:HookScript(nameplateFrame, "OnUpdate", "OnUpdate")
		end
	--end

	--[[
	self:Hook(bGlowRegion, "Show", "ShowFlash", true)
	self:Hook(bGlowRegion, "Hide", "HideFlash", true)
	self:Hook(bGlowRegion, "SetVertexColor", "FlashColor", true)
	self:Hook(bGlowRegion, "SetAlpha", "FlashAlpha", true)
	self:Hook(nameplateFrame, "SetBackdrop", "NameplateBackdrop", true)
	-- TODO: also try SetBlendMode, SetAllPoints
	]]
end

local elapsed = 0 -- accumulator for time elapsed since last glow update
function AloftGlow:OnUpdate(this, seconds)
	if (profile.enable) then
		elapsed = elapsed + seconds -- we have fractional seconds since the last invocation, so we can accumulate
		if (elapsed >= profile.interval) then
			elapsed = 0

			local aloftData = this.aloftData
			--if (not aloftData.unitid) then -- if not a groupmember nameplate
				local bGlowRegion = aloftData.bGlowRegion -- Blizzard glow
				-- local glowFrame = aloftData.glowFrame -- Aloft glow
				local healthBar = aloftData.healthBar
				-- TODO: look into doing this more efficiently, since it is "OnUpdate" processing
				-- slave the Aloft glow to the Blizzard glow; the Blizzard glow now has an "empty" texture, but it is still there, we can obtain its visibility and color, as modified by the WoW client)
				--[[
				if (bGlowRegion:IsShown() and not glowFrame:IsShown()) then
					--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "OnUpdate", 5, "glow frame show " .. tostring(aloftData.name)) end
					glowFrame:Show()
				elseif (not bGlowRegion:IsShown() and glowFrame:IsShown()) then
					--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "OnUpdate", 5, "glow frame hide " .. tostring(aloftData.name)) end
					glowFrame:Hide()
				end
				]]
				if (bGlowRegion:IsShown()) then
					--local glowRegion = glowFrame.glowRegion
					bGlowRegion:SetTexture(glowTexture[profile.style])

					bGlowRegion:ClearAllPoints()
					bGlowRegion:SetPoint("TOPLEFT", healthBar, "TOPLEFT", profile.offsetX, profile.offsetY)
					bGlowRegion:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", profile.offsetX, profile.offsetY)
					--bGlowRegion:SetPoint("CENTER", this, "CENTER", profile.offsetX, profile.offsetY)

					bGlowRegion:SetWidth(profile.width)
					bGlowRegion:SetHeight(profile.height)

					if (profile.override) then
						bGlowRegion:SetVertexColor(unpack(profile.color)) -- override the underlying blizzard color(s)
					end

					bGlowRegion:SetBlendMode(profile.blendMode)
				end
			end
			--end
			--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "OnUpdate", 5, "low visible " .. tostring(aloftData.name) .. tostring(glowFrame.glowRegion:IsVisible())) end
		end
	end
	self.hooks[this]["OnUpdate"](this, seconds) -- *must* always chain every event, or the nameplate flickers
end

function AloftGlow:ShowFlash(this)
	local aloftData = this.aloftData
	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "ShowFlash", 5, "flash show " .. tostring(this.aloftData.name)) end
	self.hooks[this].Show(this)
end

function AloftGlow:HideFlash(this)
	local aloftData = this.aloftData
	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "HideFlash", 5, "flash hide " .. tostring(this.aloftData.name)) end
	self.hooks[this].Hide(this)
end

function AloftGlow:FlashColor(this, r, g, b, a)
	local aloftData = this.aloftData
	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "FlashColor", 5, "flash color " .. tostring(this.aloftData.name)) end
	self.hooks[this].SetVertexColor(this, r, g, b, a)
end

function AloftGlow:FlashAlpha(this, a)
	local aloftData = this.aloftData
	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "FlashAlpha", 5, "flash alpha " .. tostring(this.aloftData.name)) end
	self.hooks[this].SetAlpha(this, a)
end

function AloftGlow:NameplateBackdrop(this, t)
	local aloftData = this.aloftData
	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "FlashAlpha", 5, "nameplate backdrop " .. tostring(this.aloftData.name)) end
	self.hooks[this].SetBackdrop(this, t)
end

function AloftGlow:AcquireGlowFrame(aloftData)
	--if (Aloft:IsLogEnbl("Glow", 2)) then Aloft:Log("Glow", "AcquireGlowFrame", 2, "acquire " .. tostring(this.aloftData.name)) end
	local glowFrame = aloftData.glowFrame
	if not glowFrame then
		glowFrame = next(glowFramePool)
		if glowFrame then
			glowFramePool[glowFrame] = nil
			glowFrame:SetParent(aloftData.nameplateFrame)
		else
			glowFrame = CreateFrame("Frame", nil, aloftData.nameplateFrame)
			self:SetupGlowFrame(glowFrame)
		end
		aloftData.glowFrame = glowFrame
		self:PlaceGlowFrame(aloftData)
	end
	return glowFrame
end

function AloftGlow:SetupGlowFrame(glowFrame)
	local glowRegion = glowFrame.glowRegion
	if (not glowRegion) then
		glowRegion = glowFrame:CreateTexture(nil, "BORDER")
		glowFrame.glowRegion = glowRegion
	end

	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "SetupGlowFrame", 5, "texture " .. tostring(glowTexture[profile.style]) .. " - " .. tostring(glowRegion:GetTexture())) end
	glowRegion:SetTexture(glowTexture[profile.style])
	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "SetupGlowFrame", 5, "texture " .. tostring(glowTexture[profile.style]) .. " - " .. tostring(glowRegion:GetTexture())) end
	glowRegion:SetBlendMode(profile.blendMode)
	--if (Aloft:IsLogEnbl("Glow", 5)) then Aloft:Log("Glow", "SetupGlowFrame", 5, "texture " .. tostring(glowRegion:GetTexture())) end

	glowRegion:ClearAllPoints()
	glowRegion:SetAllPoints(glowFrame) -- conform to the underlying glow frame
	glowRegion:Show()

	local frameProfile = Aloft:AcquireDBNamespace("frame").profile
	local healthBarProfile = Aloft:AcquireDBNamespace("healthBar").profile
	glowFrame:SetWidth(profile.width)		-- make the frame it slightly larger than the underlying health bar frame
	glowFrame:SetHeight(profile.height)	-- make the frame it slightly larger than the underlying health bar frame
	glowFrame:Hide()
end

function AloftGlow:PlaceGlowFrame(aloftData)
	local glowFrame = aloftData.glowFrame

	if (glowFrame) then
		--if (Aloft:IsLogEnbl("Glow", 2)) then Aloft:Log("Glow", "PlaceGlowFrame", 2, "place " .. tostring(aloftData.name)) end
		glowFrame:ClearAllPoints()
		glowFrame:SetPoint("CENTER", aloftData.healthBar, "CENTER", profile.offsetX, profile.offsetY)
		glowFrame:SetFrameLevel(1)
	end
end

function AloftGlow:ReleaseGlowFrame(aloftData)
	--if (Aloft:IsLogEnbl("Glow", 2)) then Aloft:Log("Glow", "ReleaseGlowFrame", 2, "release " .. tostring(aloftData.name)) end
	self:CleanupGlowFrame(aloftData)
	self:RePoolGlowFrame(aloftData)
end

function AloftGlow:CleanupGlowFrame(aloftData)
	local glowFrame = aloftData.glowFrame
	if glowFrame then
		-- no cleanup required on glow frames
	end
end

function AloftGlow:RePoolGlowFrame(aloftData)
	local glowFrame = aloftData.glowFrame
	if glowFrame then
		glowFrame:ClearAllPoints()
		glowFramePool[glowFrame] = true
		aloftData.glowFrame = nil
		glowFrame:Hide()
	end
end

-----------------------------------------------------------------------------
