local Aloft = Aloft
if not Aloft then return end

-----------------------------------------------------------------------------

local AloftIsPlayerData = Aloft:NewModule("IsPlayerData", "AceEvent-2.0")
local L = AceLibrary("AceLocale-2.2"):new("AloftIsPlayerData")

-----------------------------------------------------------------------------

AloftIsPlayerData.db = Aloft:AcquireDBNamespace("isPlayerData")
Aloft:RegisterDefaults("isPlayerData", "server", {	playerBlacklistData = { } })
Aloft:RegisterDefaults("isPlayerData", "profile", { save = false })

------------------------------------------------------------------------------

local playerBlacklistData = { }

----------------------------------------------------------------------------

Aloft.TagData.IsPlayer =
{
	data = "isPlayer",
	events = "Aloft:OnIsPlayerDataChanged",
	tableData = "isPlayer"
}

-----------------------------------------------------------------------------

local updateRequired = false
local function AddData(name, isPlayer)
	-- add to the blacklist if this is not a player but can pass as a player name
	if name and AloftIsPlayerData:IsPlayerName(name) then
		playerBlacklistData[name] = true
		updateRequired = true
	end
	return true
end

-----------------------------------------------------------------------------

function AloftIsPlayerData:OnEnable()
	if self.db.profile.save then
		playerBlacklistData = self.db.server.playerBlacklistData
	end
end

-----------------------------------------------------------------------------

function AloftIsPlayerData.SetSaveData(v)
	AloftIsPlayerData.db.profile.save = v
	AloftIsPlayerData.db.server.playerBlacklistData = (v and playerBlacklistData) or { }
end

function AloftIsPlayerData:ProvidesData()
	return "isPlayer"
end

function AloftIsPlayerData:EnableDataSource()
	-- Events to update our player name blacklist
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "UpdatePlayerBlacklistFromMouseOver")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdatePlayerBlacklistFromTarget")
end

-----------------------------------------------------------------------------

function AloftIsPlayerData:UpdateData(aloftData)
	-- This is so that pets named the same as players we've encountered aren't given a class incorrectly
	self:UpdateNameplate(aloftData, (not playerBlackistData[aloftData.name]) or true)
end

-----------------------------------------------------------------------------

function AloftIsPlayerData:UpdateVisibleNameplates()
	if updateRequired then
		for aloftData in Aloft:IterateVisibleNameplates() do
			self:UpdateNameplate(aloftData, (not playerBlackistData[aloftData.name]) or true)
		end
	end
	updateRequired = false
end

-----------------------------------------------------------------------------

function AloftIsPlayerData:UpdatePlayerBlacklistFromMouseOver()
	if not UnitIsPlayer("mouseover") then
		local name = UnitName("mouseover")
		AddData(name) -- add to the blacklist if this is not a player but can pass as a player name
		self:UpdateVisibleNameplates()
	end
end

function AloftIsPlayerData:UpdatePlayerBlacklistFromTarget()
	if not UnitIsPlayer("target") then
		local name = UnitName("target")
		AddData(name) -- add to the blacklist if this is not a player but can pass as a player name
		self:UpdateVisibleNameplates()
	end
end

-----------------------------------------------------------------------------

-- player name syntax parsing, courtesy of Rum/Stuck
function AloftIsPlayerData:IsPlayerName(name)
	-- if the name is already on the blacklist, return
	if playerBlacklistData[name] then return true end

	-- parse the name to see if it corresponds to the rules for a player name
    local caps = {}
    caps[1] = string.find(name, "%u")
    if caps[1] then
        caps[2] = string.find(name, "%u", caps[1]+1)
        if caps[2] then return end
    else
        return
    end
    if string.find(name, "[%p%s]") then
        return
    end
    return true
end

-----------------------------------------------------------------------------

function AloftIsPlayerData:UpdateNameplate(aloftData, isPlayer)
	-- we can now tell the difference between players and pets of the same name, if the ShowClassColorInVKey CVar is enabled;
	-- try to tighten up the checking, so we bomb out on a nameplate as quickly as possible
	if isPlayer and self:ValidateNameplate(aloftData, isPlayer) then
		self:AssignValues(aloftData, true)
	else
		self:AssignValues(aloftData, nil)
	end
end

function AloftIsPlayerData:ValidateNameplate(aloftData, isPlayer)
	if aloftData.type == "friendlyPlayer" or
	  ((aloftData.type == "hostilePlayer" or aloftData.type == "hostile") and
	  ((aloftData.class and aloftData.race) or (Aloft.showClassColorInVKey and Aloft:GetClassByColor(aloftData.originalHealthBarR, aloftData.originalHealthBarG, aloftData.originalHealthBarB))) and self:IsPlayerName(aloftData.name)) then
		return true
	end
	return false
end

function AloftIsPlayerData:AssignValues(aloftData, isPlayer)
	if aloftData.isPlayer ~= isPlayer then
		aloftData.isPlayer = isPlayer
		self:TriggerEvent("Aloft:OnIsPlayerDataChanged", aloftData)
	end

	if (isPlayer and aloftData.creatureType) then
		aloftData.creatureType = nil
		self:TriggerEvent("Aloft:OnCreatureTypeDataChanged", aloftData)
	end
	if (isPlayer and aloftData.classification) then
		aloftData.classification = nil
		self:TriggerEvent("Aloft:OnClassificationDataChanged", aloftData)
	end
	if (isPlayer and aloftData.isPet) then
		aloftData.isPet = nil
		self:TriggerEvent("Aloft:OnIsPetDataChanged", aloftData)
	end
	if (isPlayer and aloftData.petOwnersName) then
		aloftData.petOwnersName = nil
		self:TriggerEvent("Aloft:OnPetOwnersNameDataChanged", aloftData)
	end
end

-----------------------------------------------------------------------------
