local Aloft = Aloft
if not Aloft then return end

-----------------------------------------------------------------------------

local AloftRoster = Aloft:NewModule("Roster", "AceEvent-2.0")

-----------------------------------------------------------------------------

local numPartyUnitID = 4
local partyUnitID =
{
	"party1", "party2", "party3", "party4",
}

local numPartyPetUnitID = 4
local partyPetUnitID =
{
	"partypet1", "partypet2", "partypet3", "partypet4",
}

local numRaidUnitID = 40
local raidUnitID =
{
	"raid1",  "raid2",  "raid3",  "raid4",  "raid5",
	"raid6",  "raid7",  "raid8",  "raid9",  "raid10",
	"raid11", "raid12", "raid13", "raid14", "raid15",
	"raid16", "raid17", "raid18", "raid19", "raid20",
	"raid21", "raid22", "raid23", "raid24", "raid25",
	"raid26", "raid27", "raid28", "raid29", "raid30",
	"raid31", "raid32", "raid33", "raid34", "raid35",
	"raid36", "raid37", "raid38", "raid39", "raid40",
}

local numRaidPetUnitID = 40
local raidPetUnitID =
{
	"raidpet1", "raidpet2", "raidpet3", "raidpet4", "raidpet5",
	"raidpet6", "raidpet7", "raidpet8", "raidpet9", "raidpet10",
	"raidpet11", "raidpet12", "raidpet13", "raidpet14", "raidpet15",
	"raidpet16", "raidpet17", "raidpet18", "raidpet19", "raidpet20",
	"raidpet21", "raidpet22", "raidpet23", "raidpet24", "raidpet25",
	"raidpet26", "raidpet27", "raidpet28", "raidpet29", "raidpet30",
	"raidpet31", "raidpet32", "raidpet33", "raidpet34", "raidpet35",
	"raidpet36", "raidpet37", "raidpet38", "raidpet39", "raidpet40",
}

local numArenaUnitID = 40
local arenaUnitID =
{
	"arena1", "arena2", "arena3", "arena4", "arena5",
}

-----------------------------------------------------------------------------

-----------------------------------------------------------------------------

local inInstance
local instanceType

local rosterUnitByGUId = { }
local rosterNameType2GUId = { }

local rosterGUId2UnitId = { }
local rosterUnitId2GUId = { }

-----------------------------------------------------------------------------

local UnitGUID				= UnitGUID
local UnitName				= UnitName
local UnitIsFriend			= UnitIsFriend
local UnitIsPlayer			= UnitIsPlayer
local UnitPlayerControlled	= UnitPlayerControlled
local IsInInstance			= IsInInstance

-----------------------------------------------------------------------------

-- determine whether a friendly unit is a player or a pet
local function getRosterType(unitId)
	if UnitIsFriend("player", unitId) then
		if UnitIsPlayer(unitId) then
			return "player"
		elseif UnitPlayerControlled(unitId) then
			return "pet"
		end
	end
	return nil
end

-- add a unit to the roster
local function addUnit(unitId, override)
	local unitGUId = UnitGUID(unitId)
	if unitGUID then
		local type = getRosterType(unitId)
		if type then
			local r = rosterUnitByGUId[unitGUID]
			if not r then
				-- record the unit
				r = { id = unitId, guid = unitGUId, name = UnitName(unitId), type = type, }
				rosterUnitByGUId[unitGUId] = r
			elseif override then
				r.id = unitId
			end

			-- record the GUId for the name/type
			if not rosterNameType2GUId[name] then
				rosterNameType2GUId[name] = { }
			end
			rosterNameType2GUId[name][type] = unitGUId

			-- record the unitid for the GUId
			if not rosterGUId2UnitId[unitGUId] then
				rosterGUId2UnitId[unitGUId] = { }
			end
			rosterGUId2UnitId[unitGUId][unitId] = true

			-- record the GUId for the unitid
			rosterUnitId2GUId[unitId] = unitGUId
		end
	end
end

-- delete a unit from the roster
local function rmvUnit(unitId, all)
	local unitGUId = rosterUnitId2GUId[unitId]
	if unitGUId then
		-- zap the unitid record of the GUId
		rosterUnitId2GUId[unitId] = nil

		if rosterGUId2UnitId[unitGUId] then
			-- zap the GUID record of the unitid
			if all then
				-- if we are zapping all record of this GUId
				rosterGUId2UnitId[unitGUId] = nil
			else
				-- if we are zapping a single unitid
				rosterGUId2UnitId[unitGUId][unitId] = nil

				-- zap the GUID unitid record if it is now empty
				local empty = true
				for _, _ in pairs(rosterGUId2UnitId[unitGUId]) do
					empty = nil
					break
				end
				if empty then
					rosterGUId2UnitId[unitGUId] = nil
				end
			end
		end

		-- there are no longer any unitid references to this GUID
		if not rosterGUId2UnitId[unitGUId] then
			local r = rosterUnitByGUId[unitGUId]

			-- zap the name/type record of the unitid
			if rosterNameType2GUId[r.name] then
				rosterNameType2GUId[r.name][r.type] = nil
			end

			-- zap the name/type if it is now empty
			local empty = true
			for _, _ in pairs(rosterNameType2GUId[r.name]) do
				empty = nil
				break
			end
			if empty then
				rosterNameType2GUId[r.name] = nil
			end

			-- zap the unit record
			rosterUnitByGUId[unitGUId] = nil
		end
	end
end

-----------------------------------------------------------------------------

function AloftRoster:RequiresData()
end

function AloftRoster:ProvidesData()
end

-----------------------------------------------------------------------------

function AloftRoster:OnInitialize()
end

function AloftRoster:OnEnable()
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "OnPartyMembersChanged")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "OnRaidRosterUpdate")
	self:RegisterEvent("UNIT_PET", "OnUnitPet")
	-- TODO: vehicle mount/dismount
	-- TODO: handle UNKNOWNOBJECT/UKNOWNBEING

	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA","OnPlayerEnteringWorld")

	inInstance, instanceType = IsInInstance()

	self:ClearToster()
	self:ScanRoster()
end

function AloftRoster:OnDisable()
	self:CancelEvent("PARTY_MEMBERS_CHANGED")
	self:CancelEvent("RAID_ROSTER_UPDATE")
	self:CancelEvent("UNIT_PET")

	self:CancelEvent("PLAYER_ENTERING_WORLD")
	self:CancelEvent("ZONE_CHANGED_NEW_AREA")
end

-----------------------------------------------------------------------------

function AloftRoster:OnPartyMembersChanged()
	self:ClearPartyRoster()
	self:ScanPartyRoster()
end

function AloftRoster:OnRaidRosterUpdate()
	self:ClearRaidRoster()
	self:ScanRaidRoster()
end

function AloftRoster:OnUnitPet(unitId)
	self:ClearPetRoster()
	self:ScanPetRoster()
end

function AloftRoster:OnPlayerEnteringWorld()
	inInstance, instanceType = IsInInstance()

	self:ClearArenaRoster()
	self:ScanArenaRoster()
end

-----------------------------------------------------------------------------

function AloftRoster:ClearRoster()
end

function AloftRoster:ClearSelfRoster()
	rmvUnit("player")
	rmvUnit("pet")
end

function AloftRoster:ClearPetRoster()
	rmvUnit("pet")
	for i = 1, 4 do
		rmvUnit(partyPetUnitID[i])
	end
	for i = 1, 40 do
		rmvUnit(raidPetUnitID[i])
	end
end

function AloftRoster:ClearPartyRoster()
	for i = 1, 4 do
		rmvUnit(partyUnitID)
		rmvUnit(partyPetUnitID[i])
	end
end

function AloftRoster:ClearRaidRoster()
	for i = 1, 40 do
		rmvUnit(raidUnitID)
		rmvUnit(raidPetUnitID[i])
	end
end

function AloftRoster:ClearArenaRoster()
	for i = 1, 5 do
		rmvUnit(arenaUnitID)
	end
end

-----------------------------------------------------------------------------

function AloftRoster:ScanRoster()
	self:ScanArenaRoster()
	if GetNumRaidMembers() > 0 then self:ScanRaidRoster() end
	if GetNumPartyMembers() > 0 then self:ScanPartyRoster() end
	self:ScanSelfRoster()
end

function AloftRoster:ScanSelfRoster()
	addUnit("player", true)
	addUnit("pet", true)
end

function AloftRoster:ScanPetRoster()
	for i = 1, 40 do
		addUnit(raidPetUnitID[i], nil)
	end
	for i = 1, 4 do
		addUnit(partyPetUnitID[i], true)
	end
	addUnit("pet", true)
end

function AloftRoster:ScanPartyRoster()
	for i = 1, 4 do
		addUnit(partyUnitID[i], true)
		addUnit(partyPetUnitID[i], true)
	end
end

function AloftRoster:ScanRaidRoster()
	for i = 1, 40 do
		addUnit(raidUnitID[i], nil)
		addUnit(raidPetUnitID[i], nil)
	end
end

function AloftRoster:ScanArenaRoster()
	if inInstance and instanceType == "arena" then
		for i = 1, 5 do
			addUnit(arenaUnitID[i], nil)
		end
	end
end

-----------------------------------------------------------------------------

-- returns the unit by name/type
function AloftRoster:GetUnitByName(name, type)
	local unitGUId = rosterNameType2UnitId[name] and rosterNameType2UnitId[name][type]
	return unitGUId and rosterUnitByUnitGUId[unitGUId]
end

-- returns the unit by unitid
function AloftRoster:GetUnitByUnitID(unitId)
	local unitGUId = rosterUnitId2GUId
	return unitGUId and rosterUnitByUnitGUId[unitGUId]
end

-- returns the unit by guid
function AloftRoster:GetUnitByGUID(unitGUId)
	return unitGUId and rosterUnitByUnitGUId[unitGUId]
end

-- returns all unitids for the given roster GUID
function AloftRoster:GetAllGUIdUnitId(unitGUId)
	return unitGUId and rosterGUId2UnitId[unitGUId]
end

-----------------------------------------------------------------------------

